import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter_widegt_wangyan/src/utils.dart';

enum ButtonType { primary, info, warning, danger }

enum LoadingType { spinner, circular }

enum ShapeType { square, round }

enum ButtonSize { large, normal, small, mini }

class WyButtonTheme {
  const WyButtonTheme(
      {this.backgroundColor,
      this.borderColor,
      this.textColor,
      this.miniPadding,
      this.smallPadding,
      this.normalPadding,
      this.largePadding});

  const WyButtonTheme.row({
    @required this.backgroundColor,
    @required this.borderColor,
    @required this.textColor,
    @required this.miniPadding,
    @required this.smallPadding,
    @required this.normalPadding,
    @required this.largePadding,
  })  : assert(backgroundColor != null),
        assert(backgroundColor != null),
        assert(backgroundColor != null),
        assert(miniPadding != null),
        assert(smallPadding != null),
        assert(normalPadding != null),
        assert(largePadding != null);

  final Color backgroundColor;
  final Color borderColor;
  final Color textColor;
  final EdgeInsets miniPadding;
  final EdgeInsets smallPadding;
  final EdgeInsets normalPadding;
  final EdgeInsets largePadding;

  WyButtonTheme copyWith({
    Color backgroundColor,
    Color borderColor,
    Color textColor,
    EdgeInsets miniPadding,
    EdgeInsets smallPadding,
    EdgeInsets normalPadding,
    EdgeInsets largePadding,
  }) {
    return WyButtonTheme.row(
        backgroundColor: backgroundColor ?? this.backgroundColor,
        borderColor: borderColor ?? this.borderColor,
        textColor: textColor ?? this.textColor,
        miniPadding: miniPadding ?? this.miniPadding,
        smallPadding: smallPadding ?? this.smallPadding,
        normalPadding: normalPadding ?? this.normalPadding,
        largePadding: largePadding ?? this.largePadding);
  }

  WyButtonTheme minxiWith(WyButtonTheme theme) {
    return WyButtonTheme.row(
        backgroundColor: theme.backgroundColor ?? this.backgroundColor,
        borderColor: theme.borderColor ?? this.borderColor,
        textColor: theme.textColor ?? this.textColor,
        miniPadding: theme.miniPadding ?? this.miniPadding,
        smallPadding: theme.smallPadding ?? this.smallPadding,
        normalPadding: theme.normalPadding ?? this.normalPadding,
        largePadding: theme.largePadding ?? this.largePadding);
  }

  static const WyButtonTheme defaultTheme = const WyButtonTheme.row(
      backgroundColor: Colors.white,
      borderColor: Color(0xffeaebec),
      textColor: Colors.black,
      miniPadding: EdgeInsets.symmetric(horizontal: 6.0, vertical: 4.0),
      smallPadding: EdgeInsets.symmetric(horizontal: 10.0, vertical: 10.0),
      normalPadding: EdgeInsets.symmetric(horizontal: 18.0, vertical: 12.0),
      largePadding: EdgeInsets.symmetric(horizontal: 24.0, vertical: 20.0));
}

// ignore: must_be_immutable
class WyButton extends StatelessWidget {
  WyButton(
      {Key key,
      this.text = '按钮',
      this.type = ButtonType.primary,
      this.plain = false,
      this.hairline = false,
      this.loading = false,
      this.block = false,
      this.shape = ShapeType.square,
      this.loadingType = LoadingType.circular,
      this.size = ButtonSize.normal,
      this.disabled = false,
      this.icon,
      this.color,
      this.buttonTheme,
      this.onClick})
      : super(key: key);
  final String text;
  final ButtonType type;
  final bool plain;
  final bool hairline;
  final bool loading;
  final bool disabled;
  final bool block;
  final LoadingType loadingType;
  final ShapeType shape;
  final IconData icon;
  final ButtonSize size;
  final Color color;
  final VoidCallback onClick;
  final WyButtonTheme buttonTheme;

  @override
  Widget build(BuildContext context) {
    return Theme(
      data:
          Theme.of(context).copyWith(buttonTheme: ButtonThemeData(minWidth: 0)),
      child: Container(
        child: FlatButton(
          padding: _edgeInsetsGeometry,
          highlightColor: Utils.setRGB(_buttonTheme.backgroundColor),
          splashColor: Colors.transparent,
          disabledColor: _buttonTheme.backgroundColor.withOpacity(0.5),
          disabledTextColor: _buttonTheme.textColor,
          color: _buttonTheme.backgroundColor,
          textColor: _buttonTheme.textColor,
          shape: RoundedRectangleBorder(
              side: BorderSide(
                  color: _buttonTheme.borderColor, width: hairline ? 1.0 : 1.5),
              borderRadius: BorderRadius.all(
                  Radius.circular(shape == ShapeType.square ? 4.0 : 26.0))),
          onPressed: onClick,
          child: Container(
            child: Row(
              mainAxisSize: _mainAxisSize,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                loadingBox(),
                iconWidget(),
                Text(
                  text,
                  style: TextStyle(fontSize: 16.0),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  /// 根据参数获取主题样式
  WyButtonTheme get _buttonTheme {
    WyButtonTheme _buttonTheme = WyButtonTheme.defaultTheme;
    Color primaryColor = Colors.white;
    switch (type) {
      case ButtonType.primary:
        primaryColor = ColorHex('#07c160');
        break;
      case ButtonType.danger:
        primaryColor = ColorHex('#ee0a24');
        break;
      case ButtonType.info:
        primaryColor = ColorHex('#1989fa');
        break;
      case ButtonType.warning:
        primaryColor = ColorHex('#ff976a');
        break;
    }
    if (color != null) {
      primaryColor = color;
      _buttonTheme = _buttonTheme.copyWith(
          backgroundColor: primaryColor,
          textColor: Colors.white,
          borderColor: Colors.transparent);
    }
    if (type != null) {
      if (plain) {
        _buttonTheme = _buttonTheme.copyWith(
            backgroundColor: Colors.white,
            textColor: primaryColor,
            borderColor: primaryColor);
      } else {
        _buttonTheme = _buttonTheme.copyWith(
            backgroundColor: primaryColor,
            textColor: Colors.white,
            borderColor: Colors.transparent);
      }
    }
    return buttonTheme != null
        ? _buttonTheme.minxiWith(buttonTheme)
        : _buttonTheme;
  }

  /// 根据size参数获取按钮的padding值
  EdgeInsets get _edgeInsetsGeometry {
    EdgeInsets result;
    switch (size) {
      case ButtonSize.large:
        result = _buttonTheme.largePadding;
        break;
      case ButtonSize.normal:
        result = _buttonTheme.normalPadding;
        break;
      case ButtonSize.small:
        result = _buttonTheme.smallPadding;
        break;
      case ButtonSize.mini:
        result = _buttonTheme.miniPadding;
        break;
    }
    return result;
  }

  ///
  MainAxisSize get _mainAxisSize {
    if (size == ButtonSize.large || block) {
      return MainAxisSize.max;
    }
    return MainAxisSize.min;
  }

  Widget loadingBox() {
    if (!loading) {
      return Container(
        width: 0,
        height: 0,
      );
    }

    Widget loadingWidget;
    if (loadingType == LoadingType.circular) {
      loadingWidget = CircularProgressIndicator(
        strokeWidth: 2,
        valueColor: new AlwaysStoppedAnimation<Color>(_buttonTheme.textColor),
      );
    } else {
      loadingWidget = CupertinoActivityIndicator();
    }
    return Container(
      width: 24.0,
      height: 24.0,
      padding: EdgeInsets.all(3.0),
      margin: EdgeInsets.only(right: 6.0),
      child: loadingWidget,
    );
  }

  Widget iconWidget() {
    if (icon == null) {
      return Container(
        width: 0,
        height: 0,
      );
    }
    return Container(
      child: Icon(
        icon,
        size: 20,
        color: plain ? _buttonTheme.backgroundColor : _buttonTheme.textColor,
      ),
      margin: EdgeInsets.only(right: 6.0),
    );
  }
}
